#!/bin/bash
# Script to generate a basic set of debian packages from cleansocks source.
# Not trying to do anything particularly fancy or standard.  Just make it work.
#http://www.king-foo.com/2011/11/creating-debianubuntu-deb-packages/
#https://ubuntuforums.org/showthread.php?t=910717

# This file is part of the Cleansocks library.  It is released under the
# Lesser GNU GPL.  See the accompanying files COPYING and COPYING.LESSER.

if ! grep -q -s cleansocks Makefile || ! [ -r cleansocks.h ] ; then
    echo Are you sure we\'re in the cleansocks source tree?
    exit 11
fi

if ! grep -i debian /etc/os-release >/dev/null 2>&1; then
    echo Are you sure this is a Debian-based distro?
    exit 12
fi

MAJOR=$(sed -n 's/^MAJOR *= *// p' Makefile)
MINOR=$(sed -n 's/^MINOR *= *// p' Makefile)
if [ -z "$MAJOR" -o -z "$MINOR" ]; then
    echo "Cannot determine cleansocks version."
    exit 1
fi
PKGREV=1

# Here's a mess.  Binary depends on openssl version, and builds differently
# for versions < 1.1.
OPENSSL_VERSION=$(dpkg -s openssl | sed -n 's/^Version: *// p')
case "$OPENSSL_VERSION" in
    0.*|1.0.*)
	OSSLDEP='openssl (< 1.1.0), libssl-dev (< 1.1.0)'
	SSLREV=a
	;;
    [1-9]*)
	OSSLDEP='openssl (>= 1.1.0), libssl-dev (>= 1.1.0)'
	SSLREV=b
	;;
    *)
	echo "Cannot determine installed openssl version."
	exit 2
	;;
esac

PVER=$MAJOR.$MINOR-$PKGREV
CSPNAME=libcleansocks
CSPKG=${CSPNAME}_$PVER
TLSPNAME=libcleantlsc
TLSPKG=${TLSPNAME}_$PVER$SSLREV

# Build the distro
if ! make clean || ! make ; then
    echo "Cleasocks build failed"
    exit 3
fi

# Set up build area.  Clean up any remains and create directory
rm -rf deb $CSPNAME*.deb $TLSPNAME*.deb
if ! mkdir deb || ! [ -d deb ]; then
    echo "Failed to setup up Debian build area"
    exit 4
fi

# Install the build.
if ! mkdir deb/$CSPKG; then 
    echo "Failed to create cleansocks mock build area"
    exit 5
fi
BUILD=$PWD/deb/$CSPKG/usr
if ! INSTALLBASE=$BUILD LDCONFIG=echo make install; then
    echo "Cleansocks build install failed."
    exit 6
fi

(cd deb
 # Separate the files for the two packages.
 mkdir $TLSPKG
 mkdir $TLSPKG/usr
 mkdir $TLSPKG/usr/lib $TLSPKG/usr/include
 mv $CSPKG/usr/include/cleantlsc.h $TLSPKG/usr/include
 mv $CSPKG/usr/lib/libcleantlsc* $TLSPKG/usr/lib

 # This is in ca-certificates in RH, but the Ubuntu
 # package of that name doesn't have this file.  Don't find it elsewhere.
 mkdir $TLSPKG/etc
 mkdir $TLSPKG/etc/cleansocks
 (cd $TLSPKG/etc/cleansocks
   wget https://curl.haxx.se/ca/cacert.pem
   mv cacert.pem ca.pem)

 # Base control
 mkdir $CSPKG/DEBIAN
 cat > $CSPKG/DEBIAN/control <<EOF
Package: $CSPNAME
Version: $PVER
Section: base
Priority: optional
Architecture: amd64
Depends: g++
Homepage: http://sandbox.mc.edu/~bennet/cleansocks
Maintainer: Tom Bennet <bennet@mc.edu>
Description: Cleaner socket interface using C++.
EOF

 # cleantlsc control
 mkdir $TLSPKG/DEBIAN
 cat > $TLSPKG/DEBIAN/control <<EOF
Package: $TLSPNAME
Version: $PVER$SSLMOD
Section: base
Priority: optional
Architecture: amd64
Depends: $OSSLDEP, $CSPNAME (>= $PVER)
Enhances: $CSPNAME
Homepage: http://sandbox.mc.edu/~bennet/cleansocks
Maintainer: Tom Bennet <bennet@mc.edu>
Description: TLS client library for Cleansocks.
EOF

 # Some more details.
 for d in $CSPKG/DEBIAN $TLSPKG/DEBIAN; do
     echo <<EOF > $d/postinst
#!/bin/bash
ldconfig
EOF
     chmod +x $d/postinst

     echo <<EOF > $d/copyright
Format: https://www.debian.org/doc/packaging-manuals/copyright-format/1.0/
Upstream-Name: Cleansocks

Files *
Copyright: 2013 and later Tom Bennet <bennet@mc.edu>
License: LGPL-3.0
EOF
done
)

# Build the Debian packages from the pseudo-install.
dpkg-deb --build deb/$CSPKG
dpkg-deb --build deb/$TLSPKG
mv deb/*.deb .

# Clean up
rm -rf deb
