/*
 * Small test of the casting and size controls on TCP and UDP sockets.  
 * Tries some good and bad, with catches so the program should not finally 
 * crash.
 */

/*
 * This file is part of the Cleansocks library.  It is released under the
 * Lesser GNU GPL.  See the accompanying files COPYING and COPYING.LESSER.
 */

#include <iostream>
#include <string>

#include "cleansocks.h"
#include "cleanip.h"

#ifndef WIN32
#include <sys/un.h>
#endif

using namespace std;
using namespace cleansocks;

int badct = 0;

void stuff(string msg, sock &a, sock &b, bool should_be_bad)
{
	bool failed = false;

	cout << "Base asssign " << msg << ": ";
	try {
		a = b;
	} catch(socket_error &e) {
		cout << "Boom: " << e.what();

		failed = true;
	}

	if(failed != should_be_bad) {
		++badct;
		cout << " ** Bad result **" << endl;
	} else
		cout << " == Ok ==" << endl;
}

template <typename T1, typename T2>
void copycons(string msg, bool should_be_bad)
{
	bool failed = false;

	cout << "Copy construct " << msg << ": ";
	try {
		T1 s1;
		T2 s2(s1);
	} catch(socket_error &e) {
		cout << "Boom: " << e.what();

		failed = true;
	}

	if(failed != should_be_bad) {
		++badct;
		cout << " ** Bad result **" << endl;
	} else
		cout << " == Ok ==" << endl;
}

template <typename T1, typename T2>
void assign(string msg, bool should_be_bad, T1 s1 = T1(), T2 s2 = T2())
{
	bool failed = false;

	cout << "Derived assign " << msg << ": ";
	try {
		//T1 s1;
		//T2 s2;
		s1 = s2;
	} catch(socket_error &e) {
		cout << "Boom: " << e.what();

		failed = true;
	}

	if(failed != should_be_bad) {
		++badct;
		cout << " ** Bad result **" << endl;
	} else
		cout << " == Ok ==" << endl;
}

int main(int argc, char **argv)
{
	{
		TCPsocket s1, s2;
		stuff("TCP Sockets", s1, s2, false);
	}
	{
		UDPsocket s1, s2;
		stuff("UDP Sockets", s1, s2, false);
	}
	{
		TCPsocket s1;
		UDPsocket s2;
		stuff("TCP/UDP Socket", s1, s2, true);
	}
	{
		TCPsocket s2;
		UDPsocket s1;
		stuff("UDP/TCP Socket", s1, s2, true);
	}
	{
		sock s1(AF_INET, SOCK_STREAM, "tcp");
		sock s2(AF_INET, SOCK_STREAM, "tcp");
		stuff("Compat socks", s1, s2, false);
	}
	{
		sock s1(AF_INET, SOCK_DGRAM, "udp");
		sock s2(AF_INET, SOCK_STREAM, "tcp");
		stuff("Incompat socks", s1, s2, true);
	}
#ifndef WIN32
	{
		sock s1(AF_UNIX, SOCK_STREAM, 0);
		sock s2(AF_INET, SOCK_STREAM, "tcp");
		stuff("Incompat socks 2", s1, s2, true);
	}
#endif
	copycons<TCPsocket,UDPsocket>("TCP from UDP", true);
	copycons<UDPsocket,UDPsocket>("UDP from UDP", false);
	copycons<UDPsocket,TCPsocket>("UDP from TCP", true);
	copycons<TCPsocket,TCPsocket>("TCP from TCP", false);
	assign<TCPsocket,UDPsocket>("TCP from UDP", true);
	assign<UDPsocket,UDPsocket>("UDP from UDP", false);
	assign<UDPsocket,TCPsocket>("UDP from TCP", true);
	assign<TCPsocket,TCPsocket>("TCP from TCP", false);
	assign<TCPsocket,sock>("TCP from compat socket", false, 
				 TCPsocket(), sock(AF_INET, SOCK_STREAM, "tcp"));
	assign<TCPsocket,sock>("TCP from incompat socket", true, 
				 TCPsocket(), sock(AF_INET, SOCK_DGRAM, "udp"));

#if 0
	{
		sock s1(1,2,1), s2(3,2,1);
		stuff("Random Sockets", s1, s2, true);
	}
	{
		sock s1(4,7,1), s2(4,7,1);
		stuff("Random Sockets", s1, s2, false);
	}

	{
		sock fred(3,4,5);
		TCPsocket barry;
		bool boom = false;
		cout << " Random to TCP: ";
		try {
			fred = barry;
		} catch(socket_error &e) {
			cout << "Boom: " << e.what() << " == Ok ==" << endl;
			boom = true;
		}

		if(!boom) {
			cout << " ** Bad result **" << endl;
			++badct;
		}
	}
#endif

	cout << "*** " << badct << " errors ***" << endl;
}
